"""
Tests for the minion-manus project.

This package contains test modules for the minion-manus project components.
"""

# Ensure pytest can discover the tests
# See https://docs.pytest.org/en/stable/pythonpath.html 